@@ -1,40 +1,99 @@
 '''
-common.py - this file is part of S3QL (http://s3ql.googlecode.com)
+common.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from ..logging import logging, QuietError, LOG_ONCE # Ensure use of custom logger class
 from abc import abstractmethod, ABCMeta
 from functools import wraps
 import time
-import codecs
-import io
 import textwrap
+import hashlib
+import struct
+import hmac
 import inspect
 import ssl
 import os
 import re
-import pickletools
-import pickle
+import threading
 
 log = logging.getLogger(__name__)
 
+class RateTracker:
+    '''
+    Maintain an average occurence rate for events over a configurable time
+    window. The rate is computed with one second resolution.
+    '''
+
+    def __init__(self, window_length):
+        if not isinstance(window_length, int):
+            raise ValueError('only integer window lengths are supported')
+
+        self.buckets = [0] * window_length
+        self.window_length = window_length
+        self.last_update = int(time.monotonic())
+        self.lock = threading.Lock()
+
+    def register(self, _not_really=False):
+        '''Register occurence of an event.
+
+        The keyword argument is for class-internal use only.
+        '''
+
+        buckets = self.buckets
+        bucket_count = len(self.buckets)
+        now = int(time.monotonic())
+
+        elapsed =  min(now - self.last_update, bucket_count)
+        for i in range(elapsed):
+            buckets[(now - i) % bucket_count] = 0
+
+        if _not_really:
+            return
+
+        with self.lock:
+            buckets[now % bucket_count] += 1
+            self.last_update = now
+
+    def get_rate(self):
+        '''Return average rate of event occurance'''
+
+        self.register(_not_really=True)
+        return sum(self.buckets) / len(self.buckets)
+
+    def get_count(self):
+        '''Return total number of events in window'''
+
+        self.register(_not_really=True)
+        return sum(self.buckets)
+
+
+# We maintain a (global) running average of temporary errors, so
+# that we can log a warning if this number becomes large. We
+# use a relatively large window to prevent bogus spikes if
+# multiple threads all have to retry after a long period of
+# inactivity.
 RETRY_TIMEOUT = 60 * 60 * 24
-def retry(method):
+def retry(method, _tracker=RateTracker(60)):
     '''Wrap *method* for retrying on some exceptions
 
     If *method* raises an exception for which the instance's
-    `is_temp_failure(exc)` method is true, the *method* is called again
-    at increasing intervals. If this persists for more than `RETRY_TIMEOUT`
-    seconds, the most-recently caught exception is re-raised.
+    `is_temp_failure(exc)` method is true, the *method* is called again at
+    increasing intervals. If this persists for more than `RETRY_TIMEOUT`
+    seconds, the most-recently caught exception is re-raised. If the
+    method defines a keyword parameter *is_retry*, then this parameter
+    will be set to True whenever the function is retried.
     '''
 
     if inspect.isgeneratorfunction(method):
         raise TypeError('Wrapping a generator function is pointless')
 
+    sig = inspect.signature(method)
+    has_is_retry = 'is_retry' in sig.parameters
+
     @wraps(method)
     def wrapped(*a, **kw):
         self = a[0]
@@ -42,6 +101,8 @@ def retry(method):
         waited = 0
         retries = 0
         while True:
+            if has_is_retry:
+                kw['is_retry'] = (retries > 0)
             try:
                 return method(*a, **kw)
             except Exception as exc:
@@ -49,6 +110,16 @@ def retry(method):
                 #pylint: disable=W0212
                 if not self.is_temp_failure(exc):
                     raise
+
+                _tracker.register()
+                rate = _tracker.get_rate()
+                if  rate > 5:
+                    log.warning('Had to retry %d times over the last %d seconds, '
+                                'server or network problem?',
+                                rate * _tracker.window_length, _tracker.window_length)
+                else:
+                    log.debug('Average retry rate: %.2f Hz', rate)
+
                 if waited > RETRY_TIMEOUT:
                     log.error('%s.%s(*): Timeout exceeded, re-raising %r exception',
                             self.__class__.__name__, method.__name__, exc)
@@ -62,8 +133,9 @@ def retry(method):
                 else:
                     log_fn = log.warning
 
-                log_fn('Encountered %s exception (%s), retrying call to %s.%s for the %d-th time...',
-                       type(exc).__name__, exc, self.__class__.__name__, method.__name__, retries)
+                log_fn('Encountered %s (%s), retrying %s.%s (attempt %d)...',
+                       type(exc).__name__, exc, self.__class__.__name__, method.__name__,
+                       retries)
 
                 if hasattr(exc, 'retry_after') and exc.retry_after:
                     log.debug('retry_after is %.2f seconds', exc.retry_after)
@@ -198,6 +270,13 @@ class AbstractBackend(object, metaclass=ABCMeta):
     def  __contains__(self, key):
         return self.contains(key)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, tb):
+        self.close()
+        return False
+
     def iteritems(self):
         for key in self.list():
             yield (key, self[key])
@@ -274,8 +353,9 @@ class AbstractBackend(object, metaclass=ABCMeta):
     def store(self, key, val, metadata=None):
         """Store data under `key`.
 
-        `metadata` can be a dict of additional attributes to store with the
-        object.
+        `metadata` can be mapping with additional attributes to store with the
+        object. Keys have to be of type `str`, values have to be of elementary
+        type (`str`, `bytes`, `int`, `float` or `bool`).
 
         If no metadata is required, one can simply assign to the subscripted
         backend instead of using this function: ``backend[key] = val`` is
@@ -319,8 +399,9 @@ class AbstractBackend(object, metaclass=ABCMeta):
         """Open object for reading
 
         Return a file-like object. Data can be read using the `read`
-        method. metadata is stored in its *metadata* attribute and can be
-        modified by the caller at will. The object must be closed explicitly.
+        method. Metadata is returned in the file-like object's *metadata*
+        attribute and can be modified by the caller at will. The object must be
+        closed explicitly.
         """
 
         pass
@@ -329,8 +410,11 @@ class AbstractBackend(object, metaclass=ABCMeta):
     def open_write(self, key, metadata=None, is_compressed=False):
         """Open object for writing
 
-        `metadata` can be an additional (pickle-able) `dict` object to store with
-        the data. Returns a file- like object. The object must be closed closed
+        `metadata` can be mapping with additional attributes to store with the
+        object. Keys have to be of type `str`, values have to be of elementary
+        type (`str`, `bytes`, `int`, `float` or `bool`).
+
+        Returns a file- like object. The object must be closed closed
         explicitly. After closing, the *get_obj_size* may be used to retrieve
         the size of the stored object (which may differ from the size of the
         written data).
@@ -407,8 +491,7 @@ class AbstractBackend(object, metaclass=ABCMeta):
 
         If `dest` already exists, it will be overwritten. If *metadata* is
         `None` metadata will be copied from the source as well, otherwise
-        *metadata* becomes the metadata for the new object and must be
-        a pickle-able `dict` instance.
+        *metadata* becomes the metadata for the new object.
 
         Copying will be done on the remote side without retrieving object data.
         """
@@ -419,7 +502,8 @@ class AbstractBackend(object, metaclass=ABCMeta):
     def update_meta(self, key, metadata):
         """Replace metadata of *key* with *metadata*
 
-        Metadata must be `dict` instance and pickle-able.
+        `metadata` must be a mapping with keys of type `str`, and values of an
+        elementary type (`str`, `bytes`, `int`, `float` or `bool`).
         """
 
         pass
@@ -429,8 +513,7 @@ class AbstractBackend(object, metaclass=ABCMeta):
 
         If `dest` already exists, it will be overwritten. If *metadata* is
         `None` metadata will be preserved, otherwise *metadata* becomes the
-        metadata for the renamed object and must be a pickle-able `dict`
-        instance.
+        metadata for the renamed object.
 
         Rename done remotely without retrieving object data.
         """
@@ -567,63 +650,49 @@ def get_proxy(ssl):
 
     return proxy
 
+def checksum_basic_mapping(metadata, key=None):
+    '''Compute checksum for mapping of elementary types
 
-SAFE_UNPICKLE_OPCODES = {'BININT', 'BININT1', 'BININT2', 'LONG1', 'LONG4',
-                         'BINSTRING', 'SHORT_BINSTRING', 'GLOBAL',
-                         'NONE', 'NEWTRUE', 'NEWFALSE', 'BINUNICODE',
-                         'BINFLOAT', 'EMPTY_LIST', 'APPEND', 'APPENDS',
-                         'LIST', 'EMPTY_TUPLE', 'TUPLE', 'TUPLE1', 'TUPLE2',
-                         'TUPLE3', 'EMPTY_DICT', 'DICT', 'SETITEM',
-                         'SETITEMS', 'POP', 'DUP', 'MARK', 'POP_MARK',
-                         'BINGET', 'LONG_BINGET', 'BINPUT', 'LONG_BINPUT',
-                         'PROTO', 'STOP', 'REDUCE'}
+    Keys of *d* must be strings. Values of *d* must be of elementary
+    type (i.e., `str`, `bytes`, `int`, `float`, `complex`, `bool` or
+    None). If there is a key named ``signature``, then it is excluded
+    from the checksum computation.
 
-SAFE_UNPICKLE_GLOBAL_NAMES = { ('__builtin__', 'bytearray'),
-                               ('__builtin__', 'set'),
-                               ('__builtin__', 'frozenset'),
-                               ('_codecs', 'encode') }
-SAFE_UNPICKLE_GLOBAL_OBJS = { bytearray, set, frozenset, codecs.encode }
-
-class SafeUnpickler(pickle.Unpickler):
-    def find_class(self, module, name):
-        if (module, name) not in SAFE_UNPICKLE_GLOBAL_NAMES:
-            raise pickle.UnpicklingError("global '%s.%s' is unsafe" %
-                                         (module, name))
-        ret = super().find_class(module, name)
-        if ret not in SAFE_UNPICKLE_GLOBAL_OBJS:
-            raise pickle.UnpicklingError("global '%s.%s' is unsafe" %
-                                         (module, name))
-        return ret
-
-
-def safe_unpickle_fh(fh, fix_imports=True, encoding="ASCII",
-                  errors="strict"):
-    '''Safely unpickle untrusted data from *fh*
-
-    *fh* must be seekable.
+    If *key* is None, compute MD5. Otherwise compute HMAC using *key*.
     '''
 
-    if not fh.seekable():
-        raise TypeError('*fh* must be seekable')
-    pos = fh.tell()
+    # In order to compute a safe checksum, we need to convert each object to a
+    # unique representation (i.e, we can't use repr(), as it's output may change
+    # in the future). Furthermore, we have to make sure that the representation
+    # is theoretically reversible, or there is a potential for collision
+    # attacks.
 
-    # First make sure that we know all used opcodes
-    for (opcode, arg, _) in pickletools.genops(fh):
-        if opcode.proto > 2 or opcode.name not in SAFE_UNPICKLE_OPCODES:
-            raise pickle.UnpicklingError('opcode %s is unsafe' % opcode.name)
+    if key is None:
+        chk = hashlib.md5()
+    else:
+        chk = hmac.new(key, digestmod=hashlib.sha256)
 
-    fh.seek(pos)
+    for mkey in sorted(metadata.keys()):
+        assert isinstance(mkey, str)
+        if mkey == 'signature':
+            continue
+        chk.update(mkey.encode('utf-8'))
+        val = metadata[mkey]
+        if isinstance(val, str):
+            val = b'\0s' + val.encode('utf-8') + b'\0'
+        elif val is None:
+            val = b'\0n'
+        elif isinstance(val, int):
+            val = b'\0i' + ('%d' % val).encode() + b'\0'
+        elif isinstance(val, bool):
+            val = b'\0t' if val else b'\0f'
+        elif isinstance(val, float):
+            val = b'\0d' + struct.pack('<d', val)
+        elif isinstance(val, (bytes, bytearray)):
+            assert len(val).bit_length() <= 32
+            val = b'\0b' + struct.pack('<I', len(val))
+        else:
+            raise ValueError("Don't know how to checksum %s instances" % type(val))
+        chk.update(val)
 
-    # Then use a custom Unpickler to ensure that we only give access to
-    # specific, whitelisted globals. Note that with the above opcodes, there is
-    # no way to trigger attribute access, so "brachiating" from a white listed
-    # object to __builtins__ is not possible.
-    return SafeUnpickler(fh, fix_imports=fix_imports,
-                         encoding=encoding, errors=errors).load()
-
-def safe_unpickle(buf, fix_imports=True, encoding="ASCII",
-                  errors="strict"):
-    '''Safely unpickle untrusted data in *buf*'''
-
-    return safe_unpickle_fh(io.BytesIO(buf), fix_imports=fix_imports,
-                            encoding=encoding, errors=errors)
+    return chk.digest()
