@@ -1,16 +1,16 @@
 '''
-backends/s3c.py - this file is part of S3QL (http://s3ql.googlecode.com)
+backends/s3c.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from ..logging import logging, QuietError # Ensure use of custom logger class
 from .. import BUFSIZE
 from .common import (AbstractBackend, NoSuchObject, retry, AuthorizationError,
                      AuthenticationError, DanglingStorageURLError, retry_generator,
-                     get_proxy, get_ssl_context, CorruptedObjectError, safe_unpickle,
+                     get_proxy, get_ssl_context, CorruptedObjectError,
                      checksum_basic_mapping)
 from ..inherit_docstrings import (copy_ancestor_docstring, prepend_ancestor_docstring,
                                   ABCDocstMeta)
@@ -32,7 +32,6 @@ import re
 import tempfile
 import time
 import urllib.parse
-import pickle
 
 C_DAY_NAMES = [ 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun' ]
 C_MONTH_NAMES = [ 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' ]
@@ -86,7 +85,8 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
     # NOTE: ! This function is also used by the swift backend !
     @copy_ancestor_docstring
     def reset(self):
-        if self.conn.response_pending() or self.conn._out_remaining:
+        if (self.conn is not None and
+            (self.conn.response_pending() or self.conn._out_remaining)):
             log.debug('Resetting state of http connection %d', id(self.conn))
             self.conn.disconnect()
 
@@ -123,7 +123,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         conn =  HTTPConnection(self.hostname, self.port, proxy=self.proxy,
                                ssl_context=self.ssl_context)
-        conn.timeout = int(self.options.get('tcp-timeout', 10))
+        conn.timeout = int(self.options.get('tcp-timeout', 20))
         return conn
 
     # This method is also used implicitly for the retry handling of
@@ -194,13 +194,15 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
     @retry
     @copy_ancestor_docstring
-    def delete(self, key, force=False):
+    def delete(self, key, force=False, is_retry=False):
         log.debug('started with %s', key)
         try:
             resp = self._do_request('DELETE', '/%s%s' % (self.prefix, key))
             self._assert_empty_response(resp)
         except NoSuchKeyError:
-            if force:
+            # Server may have deleted the object even though we did not
+            # receive the response.
+            if force or is_retry:
                 pass
             else:
                 raise NoSuchObject(key)
@@ -381,7 +383,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         if extra_headers is not None:
             headers.update(extra_headers)
         headers[self.hdr_prefix + 'copy-source'] = \
-            '/%s/%s%s' % (self.bucket_name, self.prefix, src)
+            urllib.parse.quote('/%s/%s%s' % (self.bucket_name, self.prefix, src))
 
         if metadata is None:
             headers[self.hdr_prefix + 'metadata-directive'] = 'COPY'
@@ -522,7 +524,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             raise HTTPError(resp.status, resp.reason, resp.headers)
 
         # If not XML, do the best we can
-        if not XML_CONTENT_RE.match(content_type):
+        if not XML_CONTENT_RE.match(content_type) or resp.length == 0:
             self.conn.discard()
             raise HTTPError(resp.status, resp.reason, resp.headers)
 
@@ -709,33 +711,8 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         '''Extract metadata from HTTP response object'''
 
         format_ = resp.headers.get('%smeta-format' % self.hdr_prefix, 'raw')
-        if format_ in ('raw', 'pickle'):
-            meta = CaseInsensitiveDict()
-            pattern = re.compile(r'^%smeta-(.+)$' % re.escape(self.hdr_prefix),
-                                 re.IGNORECASE)
-            for fname in resp.headers:
-                hit = pattern.search(fname)
-                if hit:
-                    meta[hit.group(1)] = resp.headers[fname]
-
-            if format_ == 'raw':
-                return meta
-
-            # format_ == pickle
-            buf = ''.join(meta[x] for x in sorted(meta) if x.lower().startswith('data-'))
-            if 'md5' in meta and md5sum_b64(buf.encode('us-ascii')) != meta['md5']:
-                log.warning('MD5 mismatch in metadata for %s', obj_key)
-                raise BadDigestError('BadDigest', 'Meta MD5 for %s does not match' % obj_key)
-            try:
-                return safe_unpickle(b64decode(buf), encoding='latin1')
-            except binascii.Error:
-                raise CorruptedObjectError('Corrupted metadata, b64decode failed')
-            except pickle.UnpicklingError as exc:
-                raise CorruptedObjectError('Corrupted metadata, pickle says: %s' % exc)
-
-        elif format_ != 'raw2': # Current
-            raise RuntimeError('Unknown metadata format %s for key %s'
-                               % (format_, obj_key))
+        if format_ != 'raw2': # Current
+            raise CorruptedObjectError('Invalid metadata format: %s' % format_)
 
         parts = []
         for i in count():
