@@ -1,9 +1,9 @@
 '''
-swift.py - this file is part of S3QL (http://s3ql.googlecode.com)
+swift.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from ..logging import logging, QuietError # Ensure use of custom logger class
@@ -151,7 +151,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
         with HTTPConnection(self.hostname, self.port, proxy=self.proxy,
                               ssl_context=ssl_context) as conn:
-            conn.timeout = int(self.options.get('tcp-timeout', 10))
+            conn.timeout = int(self.options.get('tcp-timeout', 20))
 
             for auth_path in ('/v1.0', '/auth/v1.0'):
                 log.debug('GET %s', auth_path)
@@ -184,7 +184,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
                 conn =  HTTPConnection(o.hostname, o.port, proxy=self.proxy,
                                        ssl_context=ssl_context)
-                conn.timeout = int(self.options.get('tcp-timeout', 10))
+                conn.timeout = int(self.options.get('tcp-timeout', 20))
                 return conn
 
             raise RuntimeError('No valid authentication path found')
@@ -379,7 +379,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
 
     @retry
     @copy_ancestor_docstring
-    def delete(self, key, force=False):
+    def delete(self, key, force=False, is_retry=False):
         if key.endswith(TEMP_SUFFIX):
             raise ValueError('Keys must not end with %s' % TEMP_SUFFIX)
         log.debug('started with %s', key)
@@ -387,13 +387,22 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             resp = self._do_request('DELETE', '/%s%s' % (self.prefix, key))
             self._assert_empty_response(resp)
         except HTTPError as exc:
-            if exc.status == 404 and not force:
+            # Server may have deleted the object even though we did not
+            # receive the response.
+            if exc.status == 404 and not (force or is_retry):
                 raise NoSuchObject(key)
             elif exc.status != 404:
                 raise
 
-    # Temporary hack. I promise to make this nicer by the next
-    # release :-).
+    # We cannot wrap the entire copy() method into a retry() decorator, because
+    # copy() issues multiple requests. If the server happens to regularly close
+    # the connection after a request (even though it was processed correctly),
+    # we'd never make any progress if we always restart from the first request.
+    # We experimented with adding a retry(fn, args, kwargs) function to wrap
+    # individual calls, but this doesn't really improve the code because we
+    # typically also have to wrap e.g. a discard() or assert_empty() call, so we
+    # end up with having to create an additional function anyway. Having
+    # anonymous blocks in Python would be very nice here.
     @retry
     def _copy_helper(self, method, path, headers):
         self._do_request(method, path, headers=headers)
