@@ -1,19 +1,18 @@
 '''
-common.py - this file is part of S3QL (http://s3ql.googlecode.com)
+common.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, QuietError # Ensure use of custom logger class
 from . import BUFSIZE, CTRL_NAME, ROOT_INODE
-from .backends import prefix_map
-from .backends.common import (CorruptedObjectError, NoSuchObject, AuthenticationError,
-          DanglingStorageURLError, AuthorizationError)
-from .backends.comprenc import ComprencBackend
 from dugong import HostnameNotResolvable
 from getpass import getpass
+from ast import literal_eval
+from base64 import b64decode, b64encode
+import binascii
 import configparser
 import re
 import stat
@@ -22,7 +21,7 @@ import traceback
 import sys
 import os
 import subprocess
-import bz2
+import pickle
 import errno
 import hashlib
 import llfuse
@@ -41,16 +40,13 @@ def bytes2path(s):
 def get_seq_no(backend):
     '''Get current metadata sequence number'''
 
+    from .backends.common import NoSuchObject
+
     seq_nos = list(backend.list('s3ql_seq_no_'))
     if not seq_nos:
         # Maybe list result is outdated
         seq_nos = [ 's3ql_seq_no_1' ]
 
-    if (seq_nos[0].endswith('.meta')
-        or seq_nos[0].endswith('.dat')):
-        raise QuietError('Old file system revision, please run `s3qladm upgrade` first.',
-                         exitcode=32)
-
     seq_nos = [ int(x[len('s3ql_seq_no_'):]) for x in seq_nos ]
     seq_no = max(seq_nos)
 
@@ -73,36 +69,6 @@ def get_seq_no(backend):
 
     return seq_no
 
-def stream_write_bz2(ifh, ofh):
-    '''Compress *ifh* into *ofh* using bz2 compression'''
-
-    compr = bz2.BZ2Compressor(9)
-    while True:
-        buf = ifh.read(BUFSIZE)
-        if not buf:
-            break
-        buf = compr.compress(buf)
-        if buf:
-            ofh.write(buf)
-    buf = compr.flush()
-    if buf:
-        ofh.write(buf)
-
-def stream_read_bz2(ifh, ofh):
-    '''Uncompress bz2 compressed *ifh* into *ofh*'''
-
-    decompressor = bz2.BZ2Decompressor()
-    while True:
-        buf = ifh.read(BUFSIZE)
-        if not buf:
-            break
-        buf = decompressor.decompress(buf)
-        if buf:
-            ofh.write(buf)
-
-    if decompressor.unused_data or ifh.read(1) != b'':
-        raise CorruptedObjectError('Data after end of bz2 stream')
-
 def is_mounted(storage_url):
     '''Try to determine if *storage_url* is mounted
 
@@ -282,25 +248,33 @@ def assert_s3ql_mountpoint(mountpoint):
 
     return ctrlfile
 
-def get_backend(options, plain=False):
+def get_backend(options, raw=False):
     '''Return backend for given storage-url
 
-    If *plain* is true, don't attempt to unlock and don't wrap into
+    If *raw* is true, don't attempt to unlock and don't wrap into
     ComprencBackend.
     '''
 
-    return get_backend_factory(options, plain)()
+    return get_backend_factory(options.storage_url, options.backend_options,
+                               options.authfile,
+                               getattr(options, 'compress', ('lzma', 2)), raw)()
 
-def get_backend_factory(options, plain=False):
+def get_backend_factory(storage_url, backend_options, authfile,
+                        compress=('lzma', 2), raw=False):
     '''Return factory producing backend objects for given storage-url
 
-    If *plain* is true, don't attempt to unlock and don't wrap into
+    If *raw* is true, don't attempt to unlock and don't wrap into
     ComprencBackend.
     '''
 
-    hit = re.match(r'^([a-zA-Z0-9]+)://', options.storage_url)
+    from .backends import prefix_map
+    from .backends.common import (CorruptedObjectError, NoSuchObject, AuthenticationError,
+                                  DanglingStorageURLError, AuthorizationError)
+    from .backends.comprenc import ComprencBackend
+
+    hit = re.match(r'^([a-zA-Z0-9]+)://', storage_url)
     if not hit:
-        raise QuietError('Unable to parse storage url "%s"' % options.storage_url,
+        raise QuietError('Unable to parse storage url "%s"' % storage_url,
                          exitcode=2)
 
     backend = hit.group(1)
@@ -310,19 +284,19 @@ def get_backend_factory(options, plain=False):
         raise QuietError('No such backend: %s' % backend, exitcode=11)
 
     # Validate backend options
-    for opt in options.backend_options.keys():
+    for opt in backend_options.keys():
         if opt not in backend_class.known_options:
             raise QuietError('Unknown backend option: %s' % opt,
                              exitcode=3)
 
     # Read authfile
     config = configparser.ConfigParser()
-    if os.path.isfile(options.authfile):
-        mode = os.stat(options.authfile).st_mode
+    if os.path.isfile(authfile):
+        mode = os.stat(authfile).st_mode
         if mode & (stat.S_IRGRP | stat.S_IROTH):
-            raise QuietError("%s has insecure permissions, aborting." % options.authfile,
+            raise QuietError("%s has insecure permissions, aborting." % authfile,
                              exitcode=12)
-        config.read(options.authfile)
+        config.read(authfile)
 
     backend_login = None
     backend_passphrase = None
@@ -336,16 +310,12 @@ def get_backend_factory(options, plain=False):
 
         pattern = getopt('storage-url')
 
-        if not pattern or not options.storage_url.startswith(pattern):
+        if not pattern or not storage_url.startswith(pattern):
             continue
 
         backend_login = getopt('backend-login') or backend_login
         backend_passphrase = getopt('backend-password') or backend_passphrase
         fs_passphrase = getopt('fs-passphrase') or fs_passphrase
-        if getopt('fs-passphrase') is None and getopt('bucket-passphrase') is not None:
-            fs_passphrase = getopt('bucket-passphrase')
-            log.warning("Warning: the 'bucket-passphrase' configuration option has been "
-                        "renamed to 'fs-passphrase'! Please update your authinfo file.")
 
     if not backend_login and backend_class.needs_login:
         if sys.stdin.isatty():
@@ -361,8 +331,8 @@ def get_backend_factory(options, plain=False):
 
     backend = None
     try:
-        backend = backend_class(options.storage_url, backend_login, backend_passphrase,
-                                options.backend_options)
+        backend = backend_class(storage_url, backend_login, backend_passphrase,
+                                backend_options)
 
         # Do not use backend.lookup(), this would use a HEAD request and
         # not provide any useful error messages if something goes wrong
@@ -394,9 +364,9 @@ def get_backend_factory(options, plain=False):
         if backend is not None:
             backend.close()
 
-    if plain:
-        return lambda: backend_class(options.storage_url, backend_login, backend_passphrase,
-                                     options.backend_options)
+    if raw:
+        return lambda: backend_class(storage_url, backend_login, backend_passphrase,
+                                     backend_options)
 
     if encrypted and not fs_passphrase:
         if sys.stdin.isatty():
@@ -409,15 +379,10 @@ def get_backend_factory(options, plain=False):
     if fs_passphrase is not None:
         fs_passphrase = fs_passphrase.encode('utf-8')
 
-    if hasattr(options, 'compress'):
-        compress = options.compress
-    else:
-        compress = ('lzma', 2)
-
     if not encrypted:
         return lambda: ComprencBackend(None, compress,
-                                    backend_class(options.storage_url, backend_login,
-                                                  backend_passphrase, options.backend_options))
+                                    backend_class(storage_url, backend_login,
+                                                  backend_passphrase, backend_options))
 
     with ComprencBackend(fs_passphrase, compress, backend) as tmp_backend:
         try:
@@ -429,8 +394,8 @@ def get_backend_factory(options, plain=False):
     # passphrase in every backend object.
     def factory():
         b = ComprencBackend(data_pw, compress,
-                            backend_class(options.storage_url, backend_login,
-                                          backend_passphrase, options.backend_options))
+                            backend_class(storage_url, backend_login,
+                                          backend_passphrase, backend_options))
         b.fs_passphrase = fs_passphrase
         return b
 
@@ -538,7 +503,7 @@ def split_by_n(seq, n):
     while seq:
         yield seq[:n]
         seq = seq[n:]
-        
+
 def handle_on_return(fn):
     '''Provide fresh ExitStack instance in `on_return` argument'''
     @functools.wraps(fn)
@@ -548,3 +513,109 @@ def handle_on_return(fn):
             kw['on_return'] = on_return
             return fn(*a, **kw)
     return wrapper
+
+def parse_literal(buf, type_spec):
+    '''Try to parse *buf* as *type_spec*
+
+    Raise `ValueError` if *buf* does not contain a valid
+    Python literal, or if the literal does not correspond
+    to *type_spec*.
+
+    Example use::
+
+      buf = b'[1, 'a', 3]'
+      parse_literal(buf, [int, str, int])
+
+    '''
+
+    try:
+        obj = literal_eval(buf.decode())
+    except UnicodeDecodeError:
+        raise ValueError('unable to decode as utf-8')
+    except (ValueError, SyntaxError):
+        raise ValueError('unable to parse as python literal')
+
+    if (isinstance(type_spec, list) and type(obj) == list
+        and [ type(x) for x in obj ] == type_spec):
+        return obj
+    elif (isinstance(type_spec, tuple) and type(obj) == tuple
+        and [ type(x) for x in obj ] == list(type_spec)):
+        return obj
+    elif type(obj) == type_spec:
+        return obj
+
+    raise ValueError('literal has wrong type')
+
+class ThawError(Exception):
+    def __str__(self):
+        return 'Malformed serialization data'
+
+def thaw_basic_mapping(buf):
+    '''Reconstruct dict from serialized representation
+
+    *buf* must be a bytes-like object as created by
+    `freeze_basic_mapping`. Raises `ThawError` if *buf* is not a valid
+    representation.
+
+    This procedure is safe even if *buf* comes from an untrusted source.
+    '''
+
+    try:
+        d = literal_eval(buf.decode('utf-8'))
+    except (UnicodeDecodeError, SyntaxError, ValueError):
+        raise ThawError()
+
+    # Decode bytes values
+    for (k,v) in d.items():
+        if not isinstance(v, bytes):
+            continue
+        try:
+            d[k] = b64decode(v)
+        except binascii.Error:
+            raise ThawError()
+
+    return d
+
+def freeze_basic_mapping(d):
+    '''Serialize mapping of elementary types
+
+    Keys of *d* must be strings. Values of *d* must be of elementary type (i.e.,
+    `str`, `bytes`, `int`, `float`, `complex`, `bool` or None).
+
+    The output is a bytestream that can be used to reconstruct the mapping. The
+    bytestream is not guaranteed to be deterministic. Look at
+    `checksum_basic_mapping` if you need a deterministic bytestream.
+    '''
+
+    els = []
+    for (k,v) in d.items():
+        if not isinstance(k, str):
+            raise ValueError('key %s must be str, not %s' % (k, type(k)))
+
+        if (not isinstance(v, (str, bytes, bytearray, int, float, complex, bool))
+            and v is not None):
+            raise ValueError('value for key %s (%s) is not elementary' % (k, v))
+
+        # To avoid wasting space, we b64encode non-ascii byte values.
+        if isinstance(v, (bytes, bytearray)):
+            v = b64encode(v)
+
+        # This should be a pretty safe assumption for elementary types, but we
+        # add an assert just to be safe (Python docs just say that repr makes
+        # "best effort" to produce something parseable)
+        (k_repr, v_repr)  = (repr(k), repr(v))
+        assert (literal_eval(k_repr), literal_eval(v_repr)) == (k, v)
+
+        els.append(('%s: %s' % (k_repr, v_repr)))
+
+    buf = '{ %s }' % ', '.join(els)
+    return buf.encode('utf-8')
+
+def load_params(fname):
+    with open(fname, 'rb') as fh:
+        proto = fh.read(2)
+        fh.seek(0)
+        if proto == b'\x80\x02':
+            return pickle.load(fh)
+        else:
+            return thaw_basic_mapping(fh.read())
