@@ -1,15 +1,15 @@
 '''
-fs.py - this file is part of S3QL (http://s3ql.googlecode.com)
+fs.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging
-from . import deltadump, CTRL_NAME, CTRL_INODE, PICKLE_PROTOCOL
+from . import deltadump, CTRL_NAME, CTRL_INODE
 from .backends.common import NoSuchObject, CorruptedObjectError
-from .common import get_path
+from .common import get_path, parse_literal
 from .database import NoSuchRowError
 from .inode_cache import OutOfInodesError
 from io import BytesIO
@@ -19,7 +19,6 @@ import errno
 import llfuse
 import math
 import os
-import pickle
 import stat
 import struct
 import time
@@ -207,7 +206,7 @@ class Operations(llfuse.Operations):
         # Handle S3QL commands
         if id_ == CTRL_INODE:
             if name == b's3ql_pid?':
-                return pickle.dumps(os.getpid(), PICKLE_PROTOCOL)
+                return ('%d' % os.getpid()).encode()
 
             elif name == b's3qlstat':
                 return self.extstat()
@@ -242,22 +241,55 @@ class Operations(llfuse.Operations):
         if id_ == CTRL_INODE:
             if name == b's3ql_flushcache!':
                 self.cache.clear()
+
             elif name == b'copy':
-                self.copy_tree(*pickle.loads(value))
+                try:
+                    tup = parse_literal(value, (int, int))
+                except ValueError:
+                    log.warning('Received malformed command via control inode')
+                    raise FUSEError.EINVAL()
+                self.copy_tree(*tup)
+
             elif name == b'upload-meta':
                 if self.upload_event is not None:
                     self.upload_event.set()
                 else:
                     raise llfuse.FUSEError(errno.ENOTTY)
+
             elif name == b'lock':
-                self.lock_tree(*pickle.loads(value))
+                try:
+                    id_ = parse_literal(value, int)
+                except ValueError:
+                    log.warning('Received malformed command via control inode')
+                    raise FUSEError.EINVAL()
+                self.lock_tree(id_)
+
             elif name == b'rmtree':
-                self.remove_tree(*pickle.loads(value))
+                try:
+                    tup = parse_literal(value, (int, bytes))
+                except ValueError:
+                    log.warning('Received malformed command via control inode')
+                    raise FUSEError.EINVAL()
+                self.remove_tree(*tup)
+
             elif name == b'logging':
-                update_logging(*pickle.loads(value))
+                try:
+                    (lvl, modules)= parse_literal(value, (str, str))
+                    lvl = logging._levelNames[lvl.upper()]
+                except (ValueError, KeyError):
+                    log.warning('Received malformed command via control inode')
+                    raise FUSEError.EINVAL()
+                update_logging(lvl, modules.split(',') if modules else None)
+
             elif name == b'cachesize':
-                self.cache.max_size = pickle.loads(value)
+                try:
+                    self.cache.cache.max_size = parse_literal(value, int)
+                except ValueError:
+                    log.warning('Received malformed command via control inode')
+                    raise FUSEError.EINVAL()
+
             else:
+                log.warning('Received unknown command via control inode')
                 raise llfuse.FUSEError(errno.EINVAL)
 
         # http://code.google.com/p/s3ql/issues/detail?id=385
@@ -829,19 +861,13 @@ class Operations(llfuse.Operations):
         if attr.st_gid is not None:
             inode.gid = attr.st_gid
 
-        if attr.st_rdev is not None:
-            inode.rdev = attr.st_rdev
-
         if attr.st_atime is not None:
             inode.atime = attr.st_atime
 
         if attr.st_mtime is not None:
             inode.mtime = attr.st_mtime
 
-        if attr.st_ctime is not None:
-            inode.ctime = attr.st_ctime
-        else:
-            inode.ctime = timestamp
+        inode.ctime = timestamp
 
         return inode
 
