@@ -1,33 +1,31 @@
 '''
-fsck.py - this file is part of S3QL (http://s3ql.googlecode.com)
+fsck.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, setup_logging, QuietError
-from . import CURRENT_FS_REV, BUFSIZE, CTRL_INODE, PICKLE_PROTOCOL, ROOT_INODE
+from . import CURRENT_FS_REV, BUFSIZE, CTRL_INODE, ROOT_INODE
 from .backends.common import NoSuchObject
 from .backends.comprenc import ComprencBackend
 from .backends.local import Backend as LocalBackend
 from .common import (inode_for_path, sha256_fh, get_path, get_backend_cachedir,
-                     get_seq_no, stream_write_bz2, stream_read_bz2, is_mounted,
-                     get_backend, pretty_print_size)
+                     get_seq_no, is_mounted, get_backend, load_params,
+                     freeze_basic_mapping)
 from .database import NoSuchRowError, Connection
-from .metadata import restore_metadata, cycle_metadata, dump_metadata, create_tables
+from .metadata import create_tables, dump_and_upload_metadata, download_metadata
 from .parse_args import ArgumentParser
 from os.path import basename
 import apsw
 import os
-import pickle
 import hashlib
 import re
 import shutil
 import itertools
 import stat
 import sys
-import tempfile
 import textwrap
 import time
 import atexit
@@ -267,7 +265,7 @@ class Fsck(object):
             (id_p_new, newname) = self.resolve_free(b"/lost+found", newname)
 
             self.log_error('Content entry for inode %d refers to non-existing name with id %d, '
-                           'moving to /lost+found/%s', inode, name_id, newname)
+                           'moving to /lost+found/%s', inode, name_id, to_str(newname))
 
             self.conn.execute('UPDATE contents SET name_id=?, parent_inode=? WHERE rowid=?',
                               (self._add_name(newname), id_p_new, rowid))
@@ -287,7 +285,8 @@ class Fsck(object):
             (id_p_new, newname) = self.resolve_free(b"/lost+found",
                                                     ('[%d]-%s' % (inode_p, name)).encode())
 
-            self.log_error('Parent inode %d for "%s" vanished, moving to /lost+found', inode_p, name)
+            self.log_error('Parent inode %d for "%s" vanished, moving to /lost+found',
+                           inode_p, to_str(name))
             self._del_name(name_id)
             self.conn.execute('UPDATE contents SET name_id=?, parent_inode=? WHERE rowid=?',
                               (self._add_name(newname), id_p_new, rowid))
@@ -308,7 +307,7 @@ class Fsck(object):
             except NoSuchRowError:
                 path = '[inode %d, parent %d]' % (inode, inode_p)
 
-            self.log_error('Inode for %s vanished, deleting', path)
+            self.log_error('Inode for %s vanished, deleting', to_str(path))
             self._del_name(name_id)
             to_delete.append(rowid)
 
@@ -329,7 +328,7 @@ class Fsck(object):
                                                 'FROM contents_v WHERE inode=?', (inode,)):
                 path = get_path(id_p, self.conn, name)
                 self.log_error('Extended attribute %d of %s refers to non-existing name %d, renaming..',
-                               rowid, path, name_id)
+                               rowid, to_str(path), name_id)
 
             while True:
                 name_id = self._add_name('lost+found_%d' % rowid)
@@ -404,7 +403,7 @@ class Fsck(object):
             (id_p, name) = self.resolve_free(b"/lost+found", name)
 
             self.log_error("Found unreachable filesystem entries, re-anchoring %s [%d] "
-                           "in /lost+found", name, inode)
+                           "in /lost+found", to_str(name), inode)
             self.conn.execute('UPDATE contents SET parent_inode=?, name_id=? '
                               'WHERE inode=? AND name_id=?',
                               (id_p, self._add_name(name), inode, name_id))
@@ -441,7 +440,7 @@ class Fsck(object):
                 self.found_errors = True
                 self.log_error("Size of inode %d (%s) does not agree with number of blocks, "
                                "setting from %d to %d",
-                               id_, get_path(id_, self.conn), size_old, size)
+                               id_, to_str(get_path(id_, self.conn)), size_old, size)
                 self.conn.execute("UPDATE inodes SET size=? WHERE id=?", (size, id_))
         finally:
             self.conn.execute('DROP TABLE min_sizes')
@@ -473,14 +472,15 @@ class Fsck(object):
                 self.found_errors = True
                 if cnt is None:
                     (id_p, name) = self.resolve_free(b"/lost+found", ("inode-%d" % id_).encode())
-                    self.log_error("Inode %d not referenced, adding as /lost+found/%s", id_, name)
+                    self.log_error("Inode %d not referenced, adding as /lost+found/%s",
+                                   id_, to_str(name))
                     self.conn.execute("INSERT INTO contents (name_id, inode, parent_inode) "
                                       "VALUES (?,?,?)", (self._add_name(basename(name)), id_, id_p))
                     self.conn.execute("UPDATE inodes SET refcount=? WHERE id=?", (1, id_))
 
                 else:
                     self.log_error("Inode %d (%s) has wrong reference count, setting from %d to %d",
-                              id_, get_path(id_, self.conn), cnt_old, cnt)
+                              id_, to_str(get_path(id_, self.conn)), cnt_old, cnt)
                     self.conn.execute("UPDATE inodes SET refcount=? WHERE id=?", (cnt, id_))
         finally:
             self.conn.execute('DROP TABLE refcounts')
@@ -504,7 +504,7 @@ class Fsck(object):
                 for (name, name_id, id_p) in self.conn.query('SELECT name, name_id, parent_inode '
                                                              'FROM contents_v WHERE inode=?', (inode,)):
                     path = get_path(id_p, self.conn, name)
-                    self.log_error("File may lack data, moved to /lost+found: %s", path)
+                    self.log_error("File may lack data, moved to /lost+found: %s", to_str(path))
                     (lof_id, newname) = self.resolve_free(b"/lost+found", escape(path))
 
                     self.conn.execute('UPDATE contents SET name_id=?, parent_inode=? '
@@ -555,7 +555,7 @@ class Fsck(object):
             for (name, name_id, id_p) in self.conn.query('SELECT name, name_id, parent_inode '
                                                          'FROM contents_v WHERE inode=?', (inode,)):
                 path = get_path(id_p, self.conn, name)
-                self.log_error("File may lack data, moved to /lost+found: %s", path)
+                self.log_error("File may lack data, moved to /lost+found: %s", to_str(path))
                 (lof_id, newname) = self.resolve_free(b"/lost+found", escape(path))
 
                 self.conn.execute('UPDATE contents SET name_id=?, parent_inode=? '
@@ -619,7 +619,8 @@ class Fsck(object):
                 elif cnt is None:
                     self.found_errors = True
                     (id_p, name) = self.resolve_free(b"/lost+found", ("block-%d" % id_).encode())
-                    self.log_error("Block %d not referenced, adding as /lost+found/%s", id_, name)
+                    self.log_error("Block %d not referenced, adding as /lost+found/%s",
+                                   id_, to_str(name))
                     timestamp = time.time()
                     size = self.conn.get_val('SELECT size FROM blocks WHERE id=?', (id_,))
                     inode = self.create_inode(mode=stat.S_IFREG | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR,
@@ -751,20 +752,20 @@ class Fsck(object):
 
                 self.found_errors = True
                 self.log_error('Inode %d (%s): directory entry has no type, changed '
-                               'to %s.', inode, get_path(inode, self.conn), made_to)
+                               'to %s.', inode, to_str(get_path(inode, self.conn)), made_to)
                 self.conn.execute('UPDATE inodes SET mode=? WHERE id=?', (mode, inode))
 
             if stat.S_ISLNK(mode) and target is None:
                 self.found_errors = True
                 self.log_error('Inode %d (%s): symlink does not have target. '
                                'This is probably going to confuse your system!',
-                               inode, get_path(inode, self.conn))
+                               inode, to_str(get_path(inode, self.conn)))
 
             if stat.S_ISLNK(mode) and target is not None and size != len(target):
                 self.found_errors = True
                 self.log_error('Inode %d (%s): symlink size (%d) does not agree with target '
                                'length (%d). This is probably going to confuse your system!',
-                               inode, get_path(inode, self.conn), size, len(target))
+                               inode, to_str(get_path(inode, self.conn)), size, len(target))
 
             if size != 0 and (not stat.S_ISREG(mode)
                               and not stat.S_ISLNK(mode)
@@ -772,33 +773,33 @@ class Fsck(object):
                 self.found_errors = True
                 self.log_error('Inode %d (%s) is not regular file but has non-zero size. '
                                'This is may confuse your system!',
-                               inode, get_path(inode, self.conn))
+                               inode, to_str(get_path(inode, self.conn)))
 
             if target is not None and not stat.S_ISLNK(mode):
                 self.found_errors = True
                 self.log_error('Inode %d (%s) is not symlink but has symlink target. '
                                'This is probably going to confuse your system!',
-                               inode, get_path(inode, self.conn))
+                               inode, to_str(get_path(inode, self.conn)))
 
             if rdev != 0 and not (stat.S_ISBLK(mode) or stat.S_ISCHR(mode)):
                 self.found_errors = True
                 self.log_error('Inode %d (%s) is not device but has device number. '
                                'This is probably going to confuse your system!',
-                               inode, get_path(inode, self.conn))
+                               inode, to_str(get_path(inode, self.conn)))
 
 
             if has_children and not stat.S_ISDIR(mode):
                 self.found_errors = True
                 self.log_error('Inode %d (%s) is not a directory but has child entries. '
                                'This is probably going to confuse your system!',
-                               inode, get_path(inode, self.conn))
+                               inode, to_str(get_path(inode, self.conn)))
 
             if (not stat.S_ISREG(mode) and
                 self.conn.has_val('SELECT 1 FROM inode_blocks WHERE inode=?', (inode,))):
                 self.found_errors = True
                 self.log_error('Inode %d (%s) is not a regular file but has data blocks. '
                                'This is probably going to confuse your system!',
-                               inode, get_path(inode, self.conn))
+                               inode, to_str(get_path(inode, self.conn)))
 
 
 
@@ -808,7 +809,8 @@ class Fsck(object):
                                             'WHERE LENGTH(name) > 255'):
             path = get_path(id_p, self.conn, name)
             self.log_error('Entry name %s... in %s has more than 255 characters, '
-                           'this could cause problems', name[:40], path[:-len(name)])
+                           'this could cause problems', to_str(name[:40]),
+                           to_str(path[:-len(name)]))
             self.found_errors = True
 
     def check_objects_refcount(self):
@@ -947,7 +949,7 @@ class Fsck(object):
                     for (name, name_id, id_p) in self.conn.query('SELECT name, name_id, parent_inode '
                                                                  'FROM contents_v WHERE inode=?', (id_,)):
                         path = get_path(id_p, self.conn, name)
-                        self.log_error("File may lack data, moved to /lost+found: %s", path)
+                        self.log_error("File may lack data, moved to /lost+found: %s", to_str(path))
                         (_, newname) = self.resolve_free(b"/lost+found", escape(path))
 
                         self.conn.execute('UPDATE contents SET name_id=?, parent_inode=? '
@@ -991,13 +993,15 @@ class Fsck(object):
 
         if not isinstance(path, bytes):
             raise TypeError('path must be of type bytes')
+        if not isinstance(name, bytes):
+            raise TypeError('name must be of type bytes')
 
         inode_p = inode_for_path(path, self.conn)
 
         # Debugging http://code.google.com/p/s3ql/issues/detail?id=217
         # and http://code.google.com/p/s3ql/issues/detail?id=261
         if len(name) > 255 - 4:
-            name = '%s ... %s' % (name[0:120], name[-120:])
+            name = b''.join((name[0:120], b' ... ', name[-120:]))
 
         i = 0
         newname = name
@@ -1046,8 +1050,7 @@ class ROFsck(Fsck):
         db = Connection(path + '.db')
         db.execute('PRAGMA journal_mode = WAL')
 
-        with open(path + '.params', 'rb') as fh:
-            param = pickle.load(fh)
+        param = load_params(path + '.params')
         super().__init__(None, None, param, db)
 
     def check(self):
@@ -1161,8 +1164,7 @@ def main(args=None):
 
     if os.path.exists(cachepath + '.params'):
         assert os.path.exists(cachepath + '.db')
-        with open(cachepath + '.params', 'rb') as fh:
-            param = pickle.load(fh)
+        param = load_params(cachepath + '.params')
         if param['seq_no'] < seq_no:
             log.info('Ignoring locally cached metadata (outdated).')
             param = backend.lookup('s3ql_metadata')
@@ -1248,25 +1250,14 @@ def main(args=None):
                              + cachepath + '.db (corrupted)\n'
                              + cachepath + '.param (intact)', exitcode=43)
     else:
-        with tempfile.TemporaryFile() as tmpfh:
-            def do_read(fh):
-                tmpfh.seek(0)
-                tmpfh.truncate()
-                stream_read_bz2(fh, tmpfh)
-
-            log.info('Downloading and decompressing metadata...')
-            backend.perform_read(do_read, "s3ql_metadata")
-
-            log.info("Reading metadata...")
-            tmpfh.seek(0)
-            db = restore_metadata(tmpfh, cachepath + '.db')
+        db = download_metadata(backend, cachepath + '.db')
 
     # Increase metadata sequence no
     param['seq_no'] += 1
     param['needs_fsck'] = True
     backend['s3ql_seq_no_%d' % param['seq_no']] = b'Empty'
     with open(cachepath + '.params', 'wb') as fh:
-        pickle.dump(param, fh, PICKLE_PROTOCOL)
+        fh.write(freeze_basic_mapping(param))
 
     fsck = Fsck(cachepath + '-cache', backend, param, db)
     fsck.check()
@@ -1291,22 +1282,9 @@ def main(args=None):
     param['last_fsck'] = time.time()
     param['last-modified'] = time.time()
 
-    log.info('Dumping metadata...')
-    with tempfile.TemporaryFile() as fh:
-        dump_metadata(db, fh)
-        def do_write(obj_fh):
-            fh.seek(0)
-            stream_write_bz2(fh, obj_fh)
-            return obj_fh
-
-        log.info("Compressing and uploading metadata...")
-        obj_fh = backend.perform_write(do_write, "s3ql_metadata_new", metadata=param,
-                                      is_compressed=True)
-    log.info('Wrote %s of compressed metadata.', pretty_print_size(obj_fh.get_obj_size()))
-    log.info('Cycling metadata backups...')
-    cycle_metadata(backend)
+    dump_and_upload_metadata(backend, db, param)
     with open(cachepath + '.params', 'wb') as fh:
-        pickle.dump(param, fh, PICKLE_PROTOCOL)
+        fh.write(freeze_basic_mapping(param))
 
     log.info('Cleaning up local metadata...')
     db.execute('ANALYZE')
@@ -1381,5 +1359,10 @@ def escape(path):
 
     return path[1:].replace(b'_', b'__').replace(b'/', b'_')
 
+def to_str(name):
+    '''Decode path name for printing'''
+
+    return str(name, encoding='utf-8', errors='replace')
+
 if __name__ == '__main__':
     main(sys.argv[1:])
