@@ -1,9 +1,9 @@
 '''
-conftest.py - this file is part of S3QL (http://s3ql.googlecode.com)
+conftest.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 
 
 This module is loaded automatically by py.test and is used to
@@ -23,6 +23,7 @@ import os.path
 import pytest
 import faulthandler
 import signal
+import gc
 
 # Converted to autouse fixture below if capture is activated
 def check_test_output(request, capfd):
@@ -92,6 +93,7 @@ def pytest_configure(config):
         warnings.simplefilter('error')
 
     # Enable faulthandler
+    global faultlog_fh
     faultlog_fh = open(os.path.join(basedir, 'tests', 'test_crit.log'), 'a')
     faulthandler.enable(faultlog_fh)
     faulthandler.register(signal.SIGUSR1, file=faultlog_fh)
@@ -131,3 +133,9 @@ def pytest_configure(config):
 
     # Make errors and warnings fatal
     s3ql.logging.EXCEPTION_SEVERITY = logging.WARNING
+
+
+# Run gc.collect() at the end of every test, so that we get ResourceWarnings
+# as early as possible.
+def pytest_runtest_teardown(item, nextitem):
+    gc.collect()
