@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 '''
-mock_server.py - this file is part of S3QL (http://s3ql.googlecode.com)
+mock_server.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from http.server import BaseHTTPRequestHandler
@@ -131,6 +131,7 @@ class S3CRequestHandler(BaseHTTPRequestHandler):
                             code='UnexpectedContent')
             return
         elif src:
+            src = urllib.parse.unquote(src)
             hit = re.match('^/([a-z0-9._-]+)/(.+)$', src)
             if not hit:
                 self.send_error(400, message='Cannot parse copy-source',
@@ -153,6 +154,7 @@ class S3CRequestHandler(BaseHTTPRequestHandler):
                     self.server.metadata[q.key] = meta
             except KeyError:
                 self.send_error(404, code='NoSuchKey', resource=src)
+                return
         else:
             data = self.rfile.read(len_)
             self.server.metadata[q.key] = meta
