@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 '''
-t1_backends.py - this file is part of S3QL (http://s3ql.googlecode.com)
+t1_backends.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 if __name__ == '__main__':
@@ -545,6 +545,27 @@ def test_copy(backend):
     assert value == value2
     assert metadata == metadata2
 
+@pytest.mark.with_backend('*/raw')
+def test_copy_special(backend):
+    key1 = 'with_+_char/orig'
+    key2 = 'with_+_char/dest'
+    value = b'Just a couple of random bytes'
+
+    backend.store(key1, value)
+
+    assert_not_in_index(backend, [key2])
+    assert_not_readable(backend, key2)
+
+    # Wait for object to become visible
+    assert_in_index(backend, [key1])
+    fetch_object(backend, key1)
+
+    backend.copy(key1, key2)
+
+    assert_in_index(backend, [key2])
+    value2 = backend[key2]
+    assert value == value2
+
 @pytest.mark.with_backend('*/raw', 'local/{aes,zlib}')
 def test_copy_newmeta(backend):
     key1 = newname()
